#include <qapplication.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qregexp.h>

#include <QTreeWidget>
#include <QTreeWidgetItem>

#include "formatsconfig.h"
#include "configuration.h"
#include "playbackbackend.h"

class FormatsListItem: public QTreeWidgetItem
{
public:
    FormatsListItem(const QString &fileExtension, const QString &mplayerArguments, QTreeWidget *parent);
    bool edit();
};

class FormatsFileExtensionValidator: public QValidator
{
public:
    FormatsFileExtensionValidator(QWidget *parent);
    QValidator::State validate(QString &input, int &) const;
    void fixup(QString &input) const;
};

// FormatsListItem

FormatsListItem::FormatsListItem(const QString &fileExtension, const QString &mplayerArguments, QTreeWidget *parent)
    : QTreeWidgetItem(parent)
{
    setText(0, fileExtension);
    setText(1, mplayerArguments);
}

bool FormatsListItem::edit()
{
    FormatsEditorDialog editDialog(this);
    bool result = editDialog.exec() == QDialog::Accepted;
    return result;
}


// FormatsFileExtensionValidator

FormatsFileExtensionValidator::FormatsFileExtensionValidator(QWidget *parent)
    : QValidator(parent)
{
}

QValidator::State FormatsFileExtensionValidator::validate(QString &input, int &) const
{
    if (input[input.length() - 1] == ',' && input[input.length() - 2] != ',')
        return QValidator::Intermediate;
    else
    {
        QRegExp re("^([a-z0-9]+,)*[a-z0-9]+$");
#ifdef QT4
        if (re.indexIn(input) == 0)
#else
        if (re.match(input) == 0)
#endif
            return QValidator::Acceptable;
        else
            return QValidator::Invalid;
    }
}

void FormatsFileExtensionValidator::fixup(QString &input) const
{
    if (input[input.length() - 1] == ',')
        input = input.left(input.length() - 1);
}


// FormatsEditorDialog

FormatsEditorDialog::FormatsEditorDialog(FormatsListItem *item, QWidget *parent)
    :	QDialog(parent),
        configIndex_(-1),
        configWidget_(NULL),
        item_(item)
{
    setModal(true);
    setWindowTitle("Format Editor");
    setSizeGripEnabled(false);

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    QGridLayout *gridLayout = new QGridLayout(this);

    QLabel *fileExtensionsLabel = new QLabel("File Extensions:", this);
    fileExtensionsLineEdit_ = new QLineEdit(this);
    fileExtensionsLineEdit_->setFixedWidth(250);
    gridLayout->addWidget(fileExtensionsLabel, 0, 0);
    gridLayout->addWidget(fileExtensionsLineEdit_, 0, 1);

    QLabel *mplayerArgumentsLabel = new QLabel("Backend to use for playback:", this);
    usePlaybackBackendComboBox_ = new QComboBox(this);
    usePlaybackBackendComboBox_->setFixedWidth(250);
    gridLayout->addWidget(mplayerArgumentsLabel, 1, 0);
    gridLayout->addWidget(usePlaybackBackendComboBox_, 1, 1);

    QPushButton *okButton = new QPushButton(tr("OK"), this);
    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));

    QPushButton *cancelButton = new QPushButton(tr("Cancel"), this);
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    configLayout_ = new QVBoxLayout(this);

    QHBoxLayout *buttonsLayout = new QHBoxLayout(this);
    buttonsLayout->setDirection(QBoxLayout::RightToLeft);
    buttonsLayout->addWidget(okButton, 0);
    buttonsLayout->addWidget(cancelButton, 0);

    mainLayout->addLayout(gridLayout);
    mainLayout->addLayout(configLayout_);
    mainLayout->addLayout(buttonsLayout);

    fileExtensionsLineEdit_->setText(item_->text(0));
    fileExtensionsLineEdit_->setValidator(new FormatsFileExtensionValidator(fileExtensionsLineEdit_));

    for (int i = 0; i < globalPlaybackBackendRegistry.factories().count(); ++i)
    {
        AbstractPlaybackBackendFactory* factory = globalPlaybackBackendRegistry.factories().at(i);
        usePlaybackBackendComboBox_->addItem(factory->friendlyDisplayName(), factory->internalClassName());
    }

    connect(usePlaybackBackendComboBox_, SIGNAL(currentIndexChanged(int)), this, SLOT(usePlaybackBackendComboBoxCurrentIndexChanged(int)));

    int factoryIndex = usePlaybackBackendComboBox_->findText(item_->text(1));
    if (factoryIndex > -1)
        usePlaybackBackendComboBox_->setCurrentIndex(factoryIndex);
    else
        usePlaybackBackendComboBox_->setCurrentIndex(0);
}

int FormatsEditorDialog::exec()
{
    int result = QDialog::exec();

    if (result == QDialog::Accepted)
    {
        QString fileExtensions = fixedUpFormatsString();
        item_->setText(0, fileExtensions);

        if (configWidget_)
        {
            configWidget_->formatsChanged(fileExtensions);
            configWidget_->saveStates();
        }

        item_->setText(1, usePlaybackBackendComboBox_->currentText());
        result = true;
    }

    return result;
}

QString FormatsEditorDialog::fixedUpFormatsString()
{
    QString fileExtensions = fileExtensionsLineEdit_->text();
    fileExtensionsLineEdit_->validator()->fixup(fileExtensions);
    return fileExtensions;
}

void FormatsEditorDialog::usePlaybackBackendComboBoxCurrentIndexChanged(int index)
{
    if (index != configIndex_)
    {
        if (configWidget_)
        {
            configLayout_->removeWidget(configWidget_);
            delete configWidget_;
            configWidget_ = NULL;
        }

        QString factoryClassName = usePlaybackBackendComboBox_->currentText();
        AbstractPlaybackBackendFactory *factory = globalPlaybackBackendRegistry.findFactory(factoryClassName);

        if (factory->supportsFormatSpecificConfigs())
        {
            configWidget_ = factory->createConfigWidget(this);
            configWidget_->setStatesByFormats(fixedUpFormatsString());
            if (configWidget_)
                configLayout_->addWidget(configWidget_);
        }
    }
}

// FormatsConfig

void FormatsConfig::setStatesByConfiguration(const Configuration& config)
{
    ConfigView::setStatesByConfiguration(config);
    readFromConfiguration(config);
}

void FormatsConfig::readFromConfiguration(const Configuration& config)
{
    list_->clear();

    for (int i = 0; i < config.formatFileExtensions.count(); ++i)
        new FormatsListItem(config.formatFileExtensions[i], config.formatAssociatedPlaybackBackends[i], list_);

    list_->resizeColumnToContents(0);
    list_->resizeColumnToContents(1);
}

void FormatsConfig::createWidgets()
{
    ConfigView::createWidgets();

    QVBoxLayout *mainLayout = new QVBoxLayout(this);

    QHBoxLayout *buttonLayout = new QHBoxLayout(this);

    QPushButton *editButton = new QPushButton(tr("Edit"), this);
    connect(editButton, SIGNAL(clicked()), this, SLOT(editSelectedFormat()));

    QPushButton *addButton = new QPushButton(tr("Add"), this);
    connect(addButton, SIGNAL(clicked()), this, SLOT(addNewFormat()));

    QPushButton *removeButton = new QPushButton(tr("Remove"), this);
    connect(removeButton, SIGNAL(clicked()), this, SLOT(removeSelectedFormat()));

    buttonLayout->addWidget(editButton);
    buttonLayout->addWidget(addButton);
    buttonLayout->addWidget(removeButton);

    mainLayout->addLayout(buttonLayout);

    // List
    list_ = new QTreeWidget(this);
    list_->setAllColumnsShowFocus(true);
    list_->setColumnCount(2);
    list_->setHeaderLabels(QStringList()
        << tr("File Extensions")
        << tr("Playback Back-End")
    );
    connect(list_, SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int)), this, SLOT(editFormat(QTreeWidgetItem *, int)));

    mainLayout->addWidget(list_);

    QPushButton *revertToDefaultFormatsButton = new QPushButton(tr("Revert to default formats"), this);
    connect(revertToDefaultFormatsButton, SIGNAL(clicked()), this, SLOT(revertToDefaultFormats()));
    mainLayout->addWidget(revertToDefaultFormatsButton);
}

void FormatsConfig::saveStatesToConfiguration()
{
    ConfigView::saveStatesToConfiguration();

    config().formatFileExtensions.clear();
    config().formatAssociatedPlaybackBackends.clear();

    for (QTreeWidgetItemIterator it(list_); *it; ++it)
    {
        FormatsListItem *item = static_cast<FormatsListItem *>(*it);
        config().formatFileExtensions.append(item->text(0));

        QString friendlyName = item->text(1);
        QString className = "";

        for (int i = 0; i < globalPlaybackBackendRegistry.factories().count(); ++i)
        {
            AbstractPlaybackBackendFactory* f = globalPlaybackBackendRegistry.factories().at(i);
            if (f->friendlyDisplayName() == friendlyName)
                className = f->internalClassName();
        }

        config().formatAssociatedPlaybackBackends.append(className);
    }

    config().updateValidExtensionsList();
}

void FormatsConfig::addNewFormat()
{
    FormatsListItem *item = new FormatsListItem("", "", list_);
    if (!item->edit())
        delete item;
}

void FormatsConfig::removeSelectedFormat()
{
    if (list_->currentItem())
        delete list_->currentItem();
}

void FormatsConfig::editSelectedFormat()
{
    if (list_->currentItem())
        editFormat(list_->currentItem(), 0);
}

void FormatsConfig::editFormat(QTreeWidgetItem *item, int column)
{
    static_cast<FormatsListItem *>(item)->edit();
}

void FormatsConfig::revertToDefaultFormats()
{
    int ret;
    ret = QMessageBox::warning(this, tr("Confirmation"),
                               tr("Revert to default formats?"),
                               tr("Revert"), tr("Cancel"), QString::null,
                               1, 1);
    if (ret == 0)
    {
        Configuration workingConfiguration_ = Configuration();

        for (int i = 0; i < globalPlaybackBackendRegistry.factories().count(); ++i)
        {
            AbstractPlaybackBackendFactory *f = globalPlaybackBackendRegistry.factories().at(i);
            if (f->supportsFormatSpecificConfigs())
                f->setFormatSpecificConfigDefaults(workingConfiguration_);
        }

        readFromConfiguration(workingConfiguration_);
    }
}
